@@ -25,6 +25,7 @@ char  * modelDir=NULL;
 char  * compDir=NULL;
 char  * calchepDir=NULL;
 int   modelNum=0;
+double BWrange=2.7;
 
 int  prepareWorkPlace(void)
 {  char * command;
@@ -229,7 +230,8 @@ numout*getMEcode(int twidth,int Gauge, char*Process, char*excludeVirtual,
    {
      *(test->cc->interface->gtwidth)=0;
      *(test->cc->interface->twidth) =0;
-     *(test->cc->interface->gswidth)=0; 
+     *(test->cc->interface->gswidth)=0;
+     *(test->cc->interface->BWrange)=BWrange; 
      free(lib_);
      return test->cc;
    }
