@@ -189,7 +189,7 @@ for(i=0;i<4;i++)
 
 /* ===========  Intergration ================ */
 
-double (*sqme)(int nsub,double GG, REAL *pvect, int * err_code)=NULL;
+double (*sqme)(int nsub,double GG, REAL *pvect, REAL*cb_coeff, int * err_code)=NULL;
 static int  nsub_stat;
 static REAL*Q=NULL;
 static REAL Pmass[5];
@@ -209,7 +209,7 @@ static double dWidthdCos(double xcos)
   if(factor==0) return 0;
   
 //printf("xcos=%e factor=%E sqme=%e\n", xcos,factor,(*sqme)(nsub_stat,GG,P,&err_code));  
-  return  factor*(*sqme)(nsub_stat,GG,P,&err_code);
+  return  factor*(*sqme)(nsub_stat,GG,P,NULL,&err_code);
 
 }
 
@@ -245,7 +245,7 @@ static double wInt14(double *x, double w)
    double res;
    res=kinematic_1_4(Pmass,x[0], x[1], 2*(x[2]-0.5),2*(x[3]-0.5),2*M_PI*x[4],pvect); 
    if(res==0) return 0;  
-   res*= (*sqme)(1,GG, pvect, &err_code);
+   res*= (*sqme)(1,GG, pvect,NULL, &err_code);
    if(err_code) return 0;
    return res*8*M_PI;
 }
@@ -269,10 +269,10 @@ static double width14(numout * cc, int * err)
 
   sqme=cc->interface->sqme;
   
-  vegPtr=vegas_init(5,50);
+  vegPtr=vegas_init(5,wInt14,50);
   
-  vegas_int(vegPtr,ncall0,alph,wInt14,&ti,&tsi); 
-  vegas_int(vegPtr,ncall0,alph,wInt14,&ti,&tsi);
+  vegas_int(vegPtr,ncall0,alph,nCORES,&ti,&tsi); 
+  vegas_int(vegPtr,ncall0,alph,nCORES,&ti,&tsi);
   vegas_finish(vegPtr);
    
   return ti;
@@ -483,7 +483,7 @@ double pWidth2(numout * cc, int nsub)
       pvect[4]=sqrt(pRestOut*pRestOut+m2*m2);
       pvect[11]=-pRestOut;
       pvect[8]=sqrt(pRestOut*pRestOut+m3*m3);
-      width = totcoef * (cc->interface->sqme)(nsub,GG,pvect,&err_code);
+      width = totcoef * (cc->interface->sqme)(nsub,GG,pvect,NULL,&err_code);
   }
   return width;
 }
@@ -526,7 +526,7 @@ static int chOpen(numout*cc, int k)
    for(j=0;j<3;j++) name[j]=cc->interface->pinf(k,j+1,NULL,pdg+j);
    s=pMass(name[0]);
    for(j=1;j<3;j++) s-=pMass(name[j]);
-   if( pdg[0]!=23 &&  abs(pdg[j])!=24 && VVdecay!=0 )
+   if( pdg[0]!=23 &&  abs(pdg[0])!=24 && VVdecay!=0 )
    for(j=1;j<3;j++) if(pdg[j]==23) s-=6; else if(abs(pdg[j])==24) s-=5;
    if(s>0) return 1; else return 0;
 }   
@@ -622,6 +622,7 @@ static double decay22List(char * pname, txtList *LL)
   sprintf(process,"%s->2*x",pname2);
   cc=getMEcode(0,ForceUG,process,NULL,"",plib);
   if(!cc) { if(LL) *LL=NULL; return -1;} 
+  passParameters(cc);
   procInfo1(cc,&ntot,NULL,NULL);
   for(wtot=0,i=1;i<=ntot;i++)  if(chOpen(cc,i))
   {     
@@ -637,7 +638,7 @@ static double decay22List(char * pname, txtList *LL)
          L_->txt=malloc(20+strlen(buff));
          strcpy(L_->txt,buff);
        } 
-       wtot+=w; 
+       wtot+=w;
     }
   }
 
@@ -704,7 +705,6 @@ static double decay22List(char * pname, txtList *LL)
       }
     }
   }
-
   if(LL)
   {  for(L_=L;L_;L_=L_->next)
      { 
@@ -787,7 +787,7 @@ double pWidth(char *name, txtList * LL)
     for(j=0;j<2;j++) if(strcmp(name,pnames[j])==0) 
     { 
       if(decayTable[i].status==1)
-      { 
+      {        
         if(LL) *LL=decayTable[i].pdList[j];
         return decayTable[i].width;
       } else if(decayTable[i].status==-1)
@@ -844,7 +844,7 @@ double pWidth(char *name, txtList * LL)
   if(Q) { Qstat=*Q; setQforParticle(Q,name);}
     
   width=decay22List(name,&L);
-  
+
   if(L) 
   {
     if(LL) *LL=L;
@@ -874,6 +874,7 @@ double pWidth(char *name, txtList * LL)
     txtList newr;
     process2Lib(l->txt ,libName);
     cc=getMEcode(0,ForceUG,l->txt,NULL,"",libName);
+    if(!cc) continue;
     if(nout==3) width=width13(cc, 1, &err); else width=width14(cc, &err);
     if(width >0)
     {
@@ -1001,7 +1002,7 @@ int slhaDecayPrint(char * name,FILE*f)
    double w;
    txtList all;
    int i,dim; 
-   long PDG;
+   int PDG;
            
    PDG=qNumbers(name,NULL,NULL,NULL);
    if(!PDG) return 0;
