@@ -22,33 +22,10 @@
 #include "parser.h"
 #include "dynamic_cs.h"
 #include "param.h"
-
-#ifdef OLD
-double aWidth(char * pName)
-{  int dim;
-   txtList LL;
-   if(!compDir)
-   { int size=100;
-     for(;;)
-     {  compDir=realloc(compDir,size+20); 
-        if(getcwd(compDir,size)) break; else size*=2;
-     }
-     strcat(compDir,"/aux");
-     libDir=malloc(strlen(compDir)+20);
-     sprintf(libDir,"%s/so_generated",compDir); 
-     modelNum=1;
-     calchepDir=getenv("CALCHEP");
-     if(!calchepDir) calchepDir=interface_ext.CALCHEP;
-     ForceUG=interface_ext.forceUG;
-   }
-
-
-   { double r=  pWidth(pName,&LL,&dim);
-/*     printf("r=%E\n",r); */
-     return r;
-   }  
-}
-#endif
+#include "vegas.h"
+#include "comp.h"
+#include "cut.h"
+#include "vp.h"
 
 static void f5_key_prog(int x)
 {
@@ -58,21 +35,26 @@ static void f5_key_prog(int x)
   while(kmenu) 
   {  
     char strmen[]="\040"
-                  " Virtual W/Z decays          OF5";
+                  " Virtual W/Z decays          OF5"
+                  " Parallelization          nCORES";  
         
     if(VVdecay)  improveStr(strmen,"OF5","ON ");
         else     improveStr(strmen,"OF5","OFF");    
+    improveStr(strmen,"nCORES","%d",nCORES);    
                                           
     menu1(20,18,"Switches",strmen,"s_switch_*",&pscr,&kmenu);
     if(kmenu==1) 
     {  VVdecay=!VVdecay;
        if(checkParam()>0){VVdecay=!VVdecay;checkParam();};
-     }     
+    } else if(kmenu==2) 
+    { char mess[40];
+      sprintf(mess,"There are %d processors. To use ", (int) sysconf(_SC_NPROCESSORS_ONLN));
+      correctInt(20,22,mess,&nCORES,1);
+    }    
   }                     
 }
 
 
-
                                                                     
 static void f6_key_prog (int x){  viewDir("."); }
                                                                         
@@ -180,9 +162,43 @@ int main(int argc,char** argv)
 /* **  initialization of the session */
   link_process(PtrInterface_ext);  
   
-  start1(title,icon,"calchep.ini;../calchep.ini",&xw_error); 
-  r_sess__(NULL); 
+  start1(title,icon,"calchep.ini;../calchep.ini",&xw_error);
+  nCORES=sysconf(_SC_NPROCESSORS_ONLN); 
 
+  if(r_sess__(NULL)==-1)
+  { 
+     char buff[200];
+     int pdg[11]={21,1,-1,2,-2,3,-3,4,-4,5,-5};  
+     int k,ok=0;
+     strcpy(buff,"{p*\\09");
+     for(k=0;k<11;k++)
+     {  char*ch=pdg2name(pdg[k]);
+        if(ch)
+        {  if(ok) strcat(buff,",");
+           strcat(buff,ch);
+           ok=1;
+        }   
+     }
+     strcat(buff,"}");
+     if(ok)
+     {  int  blind0=blind;
+        char*inkeyString0=inkeyString;    
+        inkeyString=buff;
+        blind=1;
+        edittable(1,4,&compTab,1,"n_comp",0);
+        blind=blind0;
+        inkeyString=inkeyString0;
+        fillCompositeArray();
+        
+        strcpy(buff,"{%\\09T(p*)\\09 50{%\\09J(p*,p*)\\09 0.5}");
+        inkeyString=buff;
+        blind=1;
+        edittable(1,4,&cutTab,1,"n_cut",0);
+        blind=blind0;
+        inkeyString=inkeyString0;
+     }   
+  }
+  
   goto_xy(10,10); print("Calculation of constraints.  Please, be patient.");
   escpressed();
   n_comphep();
