@@ -14,7 +14,7 @@ typedef struct hadron
 {
    char        name[P_NAME_SIZE];
    shortstr    contents;
-   int         pow;
+   int         len;
    int         parton[1000];
    int         polarized[1000];
 }  hadron;
