@@ -11,14 +11,16 @@ extern void sq_diagramsinfo(void);
 extern void viewsqdiagr(void);
 extern void sqdiagrmenu(void);
 extern void viewfeyndiag(int  del_mode);
-extern int  viewresults(int toDelete);
+extern int  viewresults(void);
 extern void showheap(void);
+extern char* currentModelName(void);
 
 extern void f3_key_prog(int x);
 extern void f4_key_prog(int x);
 extern void f5_key_prog(int x);
 extern void f6_key_prog(int x);
 extern void f9_key_prog(int x);
+extern void f10_key_prog(int x);
 
            						
 extern void editModel(int edit);
